# Standard library imports
import re
import os

# Third party imports
import flask

# Local application imports
from src.opengeodeweb_back import geode_functions, utils_functions


def test_increment_request_counter(app_context):
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    utils_functions.increment_request_counter(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1


def test_decrement_request_counter(app_context):
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1
    utils_functions.decrement_request_counter(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0


def test_update_last_request_time(app_context):
    LAST_REQUEST_TIME = flask.current_app.config.get("LAST_REQUEST_TIME")
    utils_functions.update_last_request_time(flask.current_app)
    assert flask.current_app.config.get("LAST_REQUEST_TIME") >= LAST_REQUEST_TIME


def test_before_request(app_context):
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    utils_functions.before_request(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1


def test_teardown_request(app_context):
    LAST_REQUEST_TIME = flask.current_app.config.get("LAST_REQUEST_TIME")
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1
    utils_functions.teardown_request(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    assert flask.current_app.config.get("LAST_REQUEST_TIME") >= LAST_REQUEST_TIME


def test_versions():
    list_packages = [
        "OpenGeode-core",
        "OpenGeode-IO",
        "OpenGeode-Geosciences",
        "OpenGeode-GeosciencesIO",
    ]
    versions = utils_functions.versions(list_packages)
    assert type(versions) is list
    for version in versions:
        assert type(version) is dict


def test_extension_from_filename():
    extension = utils_functions.extension_from_filename("test.toto")
    assert type(extension) is str
    assert extension.count(".") == 0


def test_handle_exception(client):
    route = "/error"
    response = client.post(route)
    assert response.status_code == 500
    data = response.get_json()
    assert type(data) is dict
    assert type(data["description"]) is str
    assert type(data["name"]) is str
    assert type(data["code"]) is int


def test_generate_native_viewable_and_light_viewable():
    geode_object = "BRep"
    folder_absolute_path = os.path.abspath("./data")
    data = geode_functions.load(
        geode_object, os.path.join(folder_absolute_path, "test.og_brep")
    )
    folder_absolute_path = "None"
    result = utils_functions.generate_native_viewable_and_light_viewable(
        geode_object, data, folder_absolute_path
    )
    assert type(result) is dict
    assert type(result["name"]) is str
    assert type(result["native_file_name"]) is str
    assert re.match(r"[0-9a-f]{32}\.[a-zA-Z0-9]+", result["native_file_name"])
    assert type(result["viewable_file_name"]) is str
    assert re.match(r"[0-9a-f]{32}\.[a-zA-Z0-9]+", result["viewable_file_name"])
    assert type(result["id"]) is str
    assert re.match(r"[0-9a-f]{32}", result["id"])
    assert type(result["object_type"]) is str
    assert type(result["binary_light_viewable"]) is str



def test_model_mesh_components(client):
    route = f"/models/vtm_component_indices"
    get_full_data = lambda: {"id": "cube"}
    json = get_full_data()
    response = client.post(route, json=json)
    assert response.status_code == 200

    uuid_dict = response.json["uuid_to_flat_index"]
    assert isinstance(uuid_dict, dict)

    indices = list(uuid_dict.values())
    indices.sort()
    assert all(indices[i] > indices[i - 1] for i in range(1, len(indices)))
    for uuid in uuid_dict.keys():
        assert isinstance(uuid, str)


def test_extract_brep_uuids(client):
    route = "/models/mesh_components"
    json_data = {"filename": "cube.og_brep", "geode_object": "BRep"}
    response = client.post(route, json=json_data)

    assert response.status_code == 200
    uuid_dict = response.json["uuid_dict"]
    assert isinstance(uuid_dict, dict)
    expected_keys = {"Block", "Line", "Surface", "Corner"}
    assert any(key in uuid_dict for key in expected_keys)
    for key, value in uuid_dict.items():
        assert isinstance(value, list)
        assert all(isinstance(v, str) for v in value)
